--[[
TODO: nothing ! ;-)
--]]

--
-- Variables
--
-- GLOBALS: GetLocale, SLASH_STINGYAUTOREPAIR1, SLASH_STINGYAUTOREPAIR2, StingyAutoRepairOptions
local DCF = DEFAULT_CHAT_FRAME;
local UnitReaction = UnitReaction;
local GetFactionInfo = GetFactionInfo;
local CanMerchantRepair = CanMerchantRepair;
local GetRepairAllCost = GetRepairAllCost;
local GetMoney = GetMoney;
local RepairAllItems = RepairAllItems;
local GetCoinTextureString = GetCoinTextureString;
local IsInGuild = IsInGuild;
local CanGuildBankRepair = CanGuildBankRepair;
local GetGuildBankWithdrawMoney = GetGuildBankWithdrawMoney;
local GetGuildBankMoney = GetGuildBankMoney;
local min,max = math.min,math.max;

--
-- Localisation
--
local repare,trop_pauvre;
local aide_onoff,aide_money,pas_de_guilde,pas_de_reparation_guilde,actif,inactif;
local use_mymoney,use_guildmoney,use_guildormymoney;
if (GetLocale() == "frFR") then
    repare = "Vos objets ont été réparés pour %s.";
    trop_pauvre = "Vous n'avez pas assez d'argent pour réparer vos objets !\n%s requis.";
    aide_onoff = "Activer/désactiver la réparation automatique (";
    aide_money = "Réparer en utilisant mon argent / celui de la guilde / celui de la guilde seulement si possible, sinon le mien (";
    pas_de_guilde = "Vous n'êtes pas dans une guilde !";
    pas_de_reparation_guilde = "Votre guilde ne vous autorise pas à utiliser son argent pour vous réparer !"
    actif = "Actif"
    inactif = "Inactif"
    use_mymoney = "Utilisation de votre argent"
    use_guildmoney = "Utilisation de l'argent de la guilde"
    use_guildormymoney = "Utilisation de l'argent de la guilde si possible, sinon le vôtre"
else
    -- for the rest, usually english since it's the default language
    repare = "Your gear has been repaired for %s.";
    trop_pauvre = "You don't have enough money to repair your gear !\n%s needed.";
    aide_onoff = "Enable/Disable autorepair"
    aide_money = "Repair using your own money / your guild's money / your guild's money if applicable, yours otherwise"
    pas_de_guilde = "You're not in a guild !";
    pas_de_reparation_guilde = "Your guild doesn't allow you to use its money to repair !"
    actif = "Enabled"
    inactif = "Disabled"
    use_mymoney = "Now using your money"
    use_guildmoney = "Now using your guild's money"
    use_guildormymoney = "Now using your guild's money if applicable, otherwise yours"
end

--
-- Constantes
--
local ChatTitle = "|cff".."00007f".."StingyAutoRepair".."|r"; -- bleu ?
local OptionC = "|cff".."ff7f00"; -- jaune ?

--
-- Vérification de la réputation
--
local MerchantMaxReputation
do
	-- Valeur max de réputation atteinte (on pourrait partir de 4 (neutre) mais ça ne change strictement rien)
	local reputationmax = 0; -- Global pour la session, la recherche se fera plus rapidement (on suppose que les réputations ne descendent pas)
	
	local reaction, factionIndex, lastFactionName
	function MerchantMaxReputation()
		-- Réputation de la faction du marchant, qui est actuellement la cible
		--reaction = UnitReaction("target", "player");
		reaction = UnitReaction("npc", "player");
	
		-- Si exalté, on ne peut pas faire mieux
		if reaction==8 then return true; end
		
		-- Si on a déjà trouvé mieux avant, on sait déjà que c'est mort
		if (reaction < reputationmax) then return false; end
	
		-- Boucle sur les factions potentielles (200 factions, ça fait de la marge)
		-- Pour avoir la réputation la plus élevée
		factionIndex = 1
		repeat
			-- Récupération des infos
			-- name, description, standingId, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfo(factionIndex)
			local name, _,standingId, _,_,_,_,_, isHeader = GetFactionInfo(factionIndex)
			if name == lastFactionName then break; end
			lastFactionName = name;
			factionIndex = factionIndex + 1;
	
			if not isHeader then
				--DEBUG("Faction:"..name..", Rep:"..standingId)
				reputationmax = max(standingId,reputationmax);
	
				-- Si y'a une réputation exaltée, on n'ira pas plus loin
				if reputationmax == 8 then break; end
			end
	
		until factionIndex > 200
	
		-- Vérification du niveau de faction (un == devrait suffire, mais ça ne coute rien de mettre un >=)
		return (reaction >= reputationmax);
	end
end

--
-- Affichage
--
local UIErrorsFrame_OnEvent = UIErrorsFrame_OnEvent;
local UIErrorsFrame = UIErrorsFrame;
local function Warn(msg)
	if not msg or msg == "" then return; end
	UIErrorsFrame_OnEvent(UIErrorsFrame, "UI_INFO_MESSAGE", msg);
end

--
-- Fonctions de réparation
--
local function MyRepair(cost)
	local mymoney = GetMoney();
	if mymoney > cost then
		RepairAllItems();
		Warn(repare:format(GetCoinTextureString(cost)));
		return true;
	else
		Warn(trop_pauvre:format(GetCoinTextureString(cost)));
	end
end

local function GuildRepair(cost)
	-- On évite les cas foireux
	if not IsInGuild() then
		Warn(pas_de_guilde);
		return false;
	end
	if not CanGuildBankRepair() then
		Warn(pas_de_reparation_guilde);
		return false;
	end
	
	-- from RepairGear
	local withdrawLimit = GetGuildBankWithdrawMoney();
	local guildBankMoney = GetGuildBankMoney();

	-- Guild leader (unlimited withdrawal privileges)
	if (withdrawLimit == -1) then
		withdrawLimit = guildBankMoney;
	else
		withdrawLimit = min(withdrawLimit, guildBankMoney);
	end

	if (cost < withdrawLimit) then
		RepairAllItems(1);
		Warn(repare:format(GetCoinTextureString(cost)));
		return true;
	end
end

--
-- Réparation automatique
--
local function StingyAutoRepair()
	if not CanMerchantRepair() then return; end

	local cost = GetRepairAllCost();
	if cost > 0 then
		--DEBUG("On doit réparer !")

		if not MerchantMaxReputation() then
			--DEBUG("La réputation envers marchand est trop basse")
			return;
		end
		
		if StingyAutoRepairOptions and StingyAutoRepairOptions.whichMoney == 1 then -- mon fric
			MyRepair(cost);
		elseif StingyAutoRepairOptions and StingyAutoRepairOptions.whichMoney == 2 then -- fric de la guilde
			GuildRepair(cost);
		else
			if not GuildRepair(cost) then MyRepair(cost); end
		end
	end
end

--
-- Frame et fonction pour réagir à l'event d'accès à un marchand
--
local function OnEvent(self,event,...)
	if event == "MERCHANT_SHOW" then
		StingyAutoRepair();
	elseif event == "ADDON_LOADED" then
		local addonName = ...;
		if addonName ~= "StingyAutoRepair" then return; end

		if not StingyAutoRepairOptions then
			StingyAutoRepairOptions = {};
		end
		if StingyAutoRepairOptions.actif == nil then
			StingyAutoRepairOptions.actif = true;
		end
		if not StingyAutoRepairOptions.whichMoney then
			-- Par défaut, on utilise son propre fric
			StingyAutoRepairOptions.whichMoney = 1;
		end
		
		-- On désactive si il le faut :
		if not StingyAutoRepairOptions.actif then
			this:Hide();
		end
	end
end

local StingyAutoRepairFrame = CreateFrame("Frame");
StingyAutoRepairFrame:SetScript("OnEvent",OnEvent);
StingyAutoRepairFrame:RegisterEvent("ADDON_LOADED");
StingyAutoRepairFrame:RegisterEvent("MERCHANT_SHOW");

--
-- Commande /
--
SLASH_STINGYAUTOREPAIR1,SLASH_STINGYAUTOREPAIR2 = "/stingyautorepair","/sar";
SlashCmdList["STINGYAUTOREPAIR"] = function(args)
	if args == "on" then
		StingyAutoRepairOptions.actif = true;
		StingyAutoRepairFrame:Show();
		DCF:AddMessage(ChatTitle.." : "..OptionC..actif.."|r");
		
	elseif args == "off" then
		StingyAutoRepairOptions.actif = false;
		StingyAutoRepairFrame:Hide();
		DCF:AddMessage(ChatTitle.." : "..OptionC..inactif.."|r");
		
	elseif args == "mymoney" then
		StingyAutoRepairOptions.whichMoney = 1;
		DCF:AddMessage(ChatTitle.." : "..OptionC..use_mymoney.."|r");
	
	elseif args == "guildmoney" then
		if not IsInGuild() then
			Warn(pas_de_guilde)
			return
		end
		if not CanGuildBankRepair() then
			Warn(pas_de_reparation_guilde)
			return
		end
		StingyAutoRepairOptions.whichMoney = 2;
		DCF:AddMessage(ChatTitle.." : "..OptionC..use_guildmoney.."|r");

	elseif args == "guildormymoney" then
		StingyAutoRepairOptions.whichMoney = 3;
		DCF:AddMessage(ChatTitle.." : "..OptionC..use_guildormymoney.."|r");

	else -- Aide
		DCF:AddMessage(ChatTitle.." : "..OptionC.."Options :|r");
		DCF:AddMessage(OptionC.."-|r "..SLASH_STINGYAUTOREPAIR1.." on/off : "..OptionC..aide_onoff ..(StingyAutoRepairOptions.actif and "on" or "off")..")|r");
		
		local etat;
		if StingyAutoRepairOptions.whichMoney == 1 then
			etat = "mymoney";
		elseif StingyAutoRepairOptions.whichMoney == 2 then
			etat = "guildmoney";
		else
			etat = "guildormymoney";
		end
		DCF:AddMessage(OptionC.."-|r "..SLASH_STINGYAUTOREPAIR1.." mymoney/guildmoney/guildormymoney : "..OptionC..aide_money..etat..")|r");
	
	end
end;
